local img, sb

-- Vertex shader code
local shader = [[
extern mat4 view;

vec4 position(mat4 transform, vec4 vertex) {
  // ignore love's transform which is only for 2d and use our own matrix
  return view * vertex;
}
]]

function love.load(args)
  img = love.graphics.newImage('image1.jpg')

  -- A spritebatch is a mesh, so let's use that for simplicity
  sb = love.graphics.newSpriteBatch(img, 64)
  -- Generate 8x8 quads (image is 640x480 which means each quad is 80x60)
  local quads = {}
  for y = 0, 7 do for x = 0, 7 do
    quads[y * 8 + x + 1] = love.graphics.newQuad(x*80, y*60, 80, 60, 640, 480)
  end end
  -- Add some quads at random
  for i = 1, 60 do
    sb:add(quads[love.math.random(1, 64)],
           love.math.random(0, 7) * 80,
           love.math.random(0, 7) * 60)
  end

  -- Compile the shader in place
  shader = love.graphics.newShader(shader)
end

local t = 0
function love.update(dt)
  t = t + dt
end

function love.draw()
  -- Use a preset matrix for demo purposes. A real matrix can be generated
  -- with a 3D math library like CPML.
  if love._version_major > 0 then
    -- love 0.11 requires the matrix in row-major order
    shader:send('view', {
       1.3, 0,    0, -450,
         0, 0, -1.7, -120,
         0, 1,    0,   50,
         0, 1,    0,   50,
    })
  else
    -- send the matrix in column-major order
    shader:send('view', {
       1.3,    0,  0,  0,
         0,    0,  1,  1,
         0, -1.7,  0,  0,
      -450, -120, 50, 50,
    })
  end

  -- Toggle vertex shader on/off every second
  if t >= 2 then
    love.graphics.setShader()
    t = 0
  elseif t >= 1 then
    love.graphics.setShader(shader)
  end

  -- Draw the spritebatch
  love.graphics.draw(sb)
end

function love.keypressed(k) if k == "escape" then love.event.quit() end end
